@@ -4,8 +4,8 @@
  * @brief common routines header
  *
  * @copyright
- * Copyright (c) 2018 - 2023 Deutsche Telekom AG.
- * Copyright (c) 2018 - 2023 CESNET, z.s.p.o.
+ * Copyright (c) 2018 - 2025 Deutsche Telekom AG.
+ * Copyright (c) 2018 - 2025 CESNET, z.s.p.o.
  *
  * This source code is licensed under BSD 3-Clause License (the "License").
  * You may not use this file except in compliance with the License.
@@ -189,7 +189,12 @@ extern const sr_module_ds_t sr_module_ds_disabled_run;
 #define SR_SHM_INITIALIZER {.fd = -1, .size = 0, .addr = NULL}
 
 /** initializer of mod_info structure */
-#define SR_MODINFO_INIT(mi, c, d, d2) memset(&(mi), 0, sizeof (mi)); (mi).ds = (d); (mi).ds2 = (d2); (mi).conn = (c)
+#define SR_MODINFO_INIT(mi, c, d, d2, op_id) \
+        memset(&(mi), 0, sizeof (mi)); \
+        (mi).ds = (d); \
+        (mi).ds2 = (d2); \
+        (mi).conn = (c); \
+        (mi).operation_id = (op_id) ? (op_id) : ATOMIC_INC_RELAXED(SR_CONN_MAIN_SHM(c)->new_operation_id)
 
 /**
  * @brief Internal information about a module to be installed.
@@ -249,11 +254,12 @@ sr_error_info_t *_sr_session_start(sr_conn_ctx_t *conn, const sr_datastore_t dat
  * @param[in] session Optional originator session.
  * @param[in] shmmod_session_del Set if @p session oper data entry should be deleted from mod SHM.
  * @param[in] timeout_ms Timeout in milliseconds.
+ * @param[in] has_change_sub_lock Currently held CHANGE SUB lock.
  * @param[in,out] err_info2 Validation errors or callback error information generated by a subscriber, if any.
  * @return err_info, NULL on success.
  */
 sr_error_info_t *sr_changes_notify_store(struct sr_mod_info_s *mod_info, sr_session_ctx_t *session, int shmmod_session_del,
-        uint32_t timeout_ms, sr_error_info_t **err_info2);
+        uint32_t timeout_ms, sr_lock_mode_t has_change_sub_lock, sr_error_info_t **err_info2);
 
 /*
  * Utility functions
@@ -1371,4 +1377,42 @@ sr_error_info_t *sr_conn_info(sr_cid_t **cids, pid_t **pids, uint32_t *count, sr
  */
 int sr_is_prod_env(void);
 
+/**
+ * @brief Generate the 'module-change' notification for newly installed modules.
+ *
+ * @param[in] conn Connection to use.
+ * @param[in] new_mods Array of new modules.
+ * @param[in] new_mod_count Count of @p new_mods.
+ */
+void sr_generate_notif_module_change_installed(sr_conn_ctx_t *conn, sr_int_install_mod_t *new_mods,
+        uint32_t new_mod_count);
+
+/**
+ * @brief Generate the 'module-change' notification for uninstalled modules.
+ *
+ * @param[in] conn Connection to use.
+ * @param[in] mod_set Set of uninstalled modules.
+ */
+void sr_generate_notif_module_change_uninstalled(sr_conn_ctx_t *conn, struct ly_set *mod_set);
+
+/**
+ * @brief Generate the 'module-change' notification for updated modules.
+ *
+ * @param[in] conn Connection to use.
+ * @param[in] old_mod_set Set of old modules.
+ * @param[in] upd_mod_set Set of updated modules.
+ */
+void sr_generate_notif_module_change_updated(sr_conn_ctx_t *conn, struct ly_set *old_mod_set, struct ly_set *upd_mod_set);
+
+/**
+ * @brief Generate the 'module-change' notification for a module feature change.
+ *
+ * @param[in] conn Connection to use.
+ * @param[in] ly_mod Changed module.
+ * @param[in] feat_set Set with all the changed features.
+ * @param[in] enabled Whether the feature was enabled or disabled.
+ */
+void sr_generate_notif_module_change_feature(sr_conn_ctx_t *conn, const struct lys_module *ly_mod,
+        const struct ly_set *feat_set, int enabled);
+
 #endif /* _COMMON_H */
