@@ -180,7 +180,16 @@ struct sr_session_ctx_s {
     char *user;                     /**< Session (system) user. */
     char *nacm_user;                /**< Optional NACM user. If set, NACM is applied. */
     sr_error_info_t *err_info;      /**< Session error information. */
-    int push_oper_data;             /**< Whether the session has stored any push operational data. */
+
+    struct sr_oper_push_cache_s {
+        char *name;                 /**< Module name whose push oper data were ever modified by this session. */
+        int has_data;               /**< Flag if there is any data from this session in the module datastore. */
+        struct lyd_node *cache;     /**< Cached operational push data from the last push.
+                                         Can be NULL even if has_data is true, for example,
+                                         after apply_changes failure or after a context change.
+                                         Protected by sr_lycc_lock from flushing. */
+    } *oper_push_mods;
+    uint32_t oper_push_mod_count;   /**< Count of modules with modified push oper data by this session. */
 
     char *orig_name;                /**< Originator name used for all events sent on this session. */
     void *orig_data;                /**< Originator data used for all events sent on this session. */
@@ -201,6 +210,7 @@ struct sr_session_ctx_s {
         sr_data_t *edit;            /**< Prepared edit data tree. */
         struct lyd_node *diff;      /**< Diff data tree, used for module change iterator. */
     } dt[SR_DS_COUNT];              /**< Session-exclusive prepared changes. */
+    int oper_edit_fetched;         /**< If current operational ds edit has been fetched, even if session->dt[SR_DS_OPERATIONAL]->edit == NULL. */
 
     struct sr_sess_notif_buf {
         int thread_running;         /**< Flag whether the notification buffering thread of this session is running. */
