@@ -50,6 +50,7 @@ struct sr_mod_info_s {
     struct lyd_node *data;      /**< Data tree. */
     int data_cached;            /**< Whether the data are actually cached. */
     sr_conn_ctx_t *conn;        /**< Associated connection. */
+    uint32_t operation_id;      /**< ID of the current operation for all the callbacks. */
 
     struct sr_mod_info_mod_s {
         sr_mod_t *shm_mod;      /**< Module SHM structure. */
@@ -228,12 +229,12 @@ void sr_modinfo_changesub_rdunlock(struct sr_mod_info_s *mod_info);
  * @brief Get specific oper DS data based on the params.
  *
  * @param[in] mod_info Mod info to use.
- * @param[in] sid If set, do not load oper data of sessions after this one, otherwise load oper data of all the sessions.
- * @param[in] oper_data Used only if @p sid is set. If set, replace the oper data of @p sid with these data, otherwise
+ * @param[in] sess Session whose oper push data should be loaded.
+ * @param[in] oper_data If set, replace the oper data of @p sess with these data, otherwise
  * use the stored data of this session.
  * @return err_info, NULL on success.
  */
-sr_error_info_t *sr_modinfo_get_oper_data(struct sr_mod_info_s *mod_info, uint32_t sid, struct lyd_node **oper_data);
+sr_error_info_t *sr_modinfo_get_oper_data(struct sr_mod_info_s *mod_info, sr_session_ctx_t *sess, struct lyd_node **oper_data);
 
 #define SR_MI_NEW_DEPS          0x01    /**< new modules are not required (MOD_INFO_REQ) but only dpendencies (MOD_INFO_DEP) */
 #define SR_MI_INV_DEPS          0x02    /**< add inverse dependencies for added modules */
@@ -340,9 +341,10 @@ sr_error_info_t *sr_modinfo_generate_config_change_notif(struct sr_mod_info_s *m
  * @param[in] mod_info Mod info to use.
  * @param[in] session Session to use, if operational DS.
  * @param[in] shmmod_session_del Set if @p session oper data entry should be deleted from mod SHM.
+ * @param[in] commit Whether to prepare to store the data or commit it.
  * @return err_info, NULL on success.
  */
-sr_error_info_t *sr_modinfo_data_store(struct sr_mod_info_s *mod_info, sr_session_ctx_t *session, int shmmod_session_del);
+sr_error_info_t *sr_modinfo_data_store(struct sr_mod_info_s *mod_info, sr_session_ctx_t *session, int shmmod_session_del, int commit);
 
 /**
  * @brief Reset (unlock SHM files) all candidate data for mod info.
