@@ -520,12 +520,13 @@ srpds_cleanup_uo_lists(srpds_db_userordered_lists_t *uo_lists)
 }
 
 sr_error_info_t *
-srpds_add_conv_mod_data(const char *plg_name, sr_datastore_t ds, const char *path, const char *name, enum srpds_db_ly_types type,
-        struct lys_module *node_module, const char *value, int32_t valtype, int *dflt_flag, const char **keys,
-        uint32_t *lengths, int64_t order, const char *path_no_pred, srpds_db_userordered_lists_t *uo_lists,
-        struct lyd_node ***parent_nodes, size_t *pnodes_size, struct lyd_node **mod_data)
+srpds_add_conv_mod_data(const char *plg_name, const struct ly_ctx *ly_ctx, sr_datastore_t ds, const char *path,
+        const char *name, enum srpds_db_ly_types type, const char *module_name, const char *value, int32_t valtype,
+        int *dflt_flag, const char **keys, uint32_t *lengths, int64_t order, const char *path_no_pred,
+        srpds_db_userordered_lists_t *uo_lists, struct lyd_node ***parent_nodes, size_t *pnodes_size, struct lyd_node **mod_data)
 {
     sr_error_info_t *err_info = NULL;
+    const struct lys_module *node_module;
     struct lyd_node *new_node = NULL, *parent_node = NULL;
     struct lyd_node **tmp_pnodes = NULL;
     uint32_t node_idx = 0;
@@ -534,11 +535,21 @@ srpds_add_conv_mod_data(const char *plg_name, sr_datastore_t ds, const char *pat
     node_idx = srpds_get_node_depth(path) - 1;
     parent_node = node_idx ? (*parent_nodes)[node_idx - 1] : NULL;
 
+    /* get the node module */
+    if (!module_name) {
+        node_module = NULL;
+    } else if (parent_node) {
+        /* use the parent context for ext data */
+        node_module = ly_ctx_get_module_implemented(LYD_CTX(parent_node), module_name);
+    } else {
+        node_module = ly_ctx_get_module_implemented(ly_ctx, module_name);
+    }
+
     /* create a node based on type */
     switch (type) {
     case SRPDS_DB_LY_CONTAINER:    /* containers */
         if (lyd_new_inner(parent_node, node_module, name, 0, &new_node) != LY_SUCCESS) {
-            ERRINFO(&err_info, plg_name, SR_ERR_LY, "lyd_new_inner()", "");
+            ERRINFO(&err_info, plg_name, SR_ERR_LY, "lyd_new_inner()", ly_last_logmsg());
             goto cleanup;
         }
         break;
@@ -546,7 +557,7 @@ srpds_add_conv_mod_data(const char *plg_name, sr_datastore_t ds, const char *pat
     case SRPDS_DB_LY_LIST_UO:  /* user-ordered lists */
         if (lyd_new_list3(parent_node, node_module, name, (const char **)keys, lengths, LYD_NEW_VAL_STORE_ONLY,
                 &new_node) != LY_SUCCESS) {
-            ERRINFO(&err_info, plg_name, SR_ERR_LY, "lyd_new_list3()", "");
+            ERRINFO(&err_info, plg_name, SR_ERR_LY, "lyd_new_list3()", ly_last_logmsg());
             goto cleanup;
         }
         break;
@@ -554,14 +565,14 @@ srpds_add_conv_mod_data(const char *plg_name, sr_datastore_t ds, const char *pat
     case SRPDS_DB_LY_LEAFLIST_UO:  /* user-ordered leaf-lists */
         if (lyd_new_term(parent_node, node_module, name, value, LYD_NEW_VAL_STORE_ONLY,
                 &new_node) != LY_SUCCESS) {
-            ERRINFO(&err_info, plg_name, SR_ERR_LY, "lyd_new_term()", "");
+            ERRINFO(&err_info, plg_name, SR_ERR_LY, "lyd_new_term()", ly_last_logmsg());
             goto cleanup;
         }
         break;
     case SRPDS_DB_LY_ANY:   /* anydata and anyxml */
         if (lyd_new_any(parent_node, node_module, name, value, valtype ? LYD_ANYDATA_JSON : LYD_ANYDATA_XML,
                 LYD_NEW_VAL_STORE_ONLY, &new_node) != LY_SUCCESS) {
-            ERRINFO(&err_info, plg_name, SR_ERR_LY, "lyd_new_any()", "");
+            ERRINFO(&err_info, plg_name, SR_ERR_LY, "lyd_new_any()", ly_last_logmsg());
             goto cleanup;
         }
         break;
@@ -581,7 +592,7 @@ srpds_add_conv_mod_data(const char *plg_name, sr_datastore_t ds, const char *pat
     (*parent_nodes)[node_idx] = new_node;
     if (!node_idx) {
         if (lyd_insert_sibling(*mod_data, new_node, mod_data) != LY_SUCCESS) {
-            ERRINFO(&err_info, plg_name, SR_ERR_LY, "lyd_insert_sibling()", "");
+            ERRINFO(&err_info, plg_name, SR_ERR_LY, "lyd_insert_sibling()", ly_last_logmsg());
             goto cleanup;
         }
     }
@@ -626,11 +637,12 @@ cleanup:
 }
 
 sr_error_info_t *
-srpds_add_oper_mod_data(const char *plg_name, struct ly_ctx *ctx, const char *path, const char *name, enum srpds_db_ly_types type,
-        const char *module_name, struct lys_module *node_module, const char *value, int32_t valtype, int *dflt_flag,
+srpds_add_oper_mod_data(const char *plg_name, const struct ly_ctx *ly_ctx, const char *path, const char *name,
+        enum srpds_db_ly_types type, const char *module_name, const char *value, int32_t valtype, int *dflt_flag,
         const char **keys, uint32_t *lengths, struct lyd_node ***parent_nodes, size_t *pnodes_size, struct lyd_node **mod_data)
 {
     sr_error_info_t *err_info = NULL;
+    const struct lys_module *node_module;
     struct lyd_node **tmp_pnodes = NULL;
     struct lyd_node *meta_match = NULL, *new_node = NULL, *parent_node = NULL;
     struct ly_set *meta_match_nodes = NULL;
@@ -652,6 +664,16 @@ srpds_add_oper_mod_data(const char *plg_name, struct ly_ctx *ctx, const char *pa
         meta_match = meta_match_nodes->dnodes[0];
     }
 
+    /* get the node module */
+    if (!module_name) {
+        node_module = NULL;
+    } else if (parent_node) {
+        /* use the parent context for ext data */
+        node_module = ly_ctx_get_module_implemented(LYD_CTX(parent_node), module_name);
+    } else {
+        node_module = ly_ctx_get_module_implemented(ly_ctx, module_name);
+    }
+
     /* create a node based on type */
     switch (type) {
     /* tree metadata (e.g. 'nc:operation="merge"' or 'or:origin="unknown"') */
@@ -696,7 +718,7 @@ srpds_add_oper_mod_data(const char *plg_name, struct ly_ctx *ctx, const char *pa
         }
         break;
     case SRPDS_DB_LY_OPAQUE:       /* opaque nodes */
-        if (lyd_new_opaq(parent_node, ctx, name, value, NULL, module_name, &new_node) != LY_SUCCESS) {
+        if (lyd_new_opaq(parent_node, ly_ctx, name, value, NULL, module_name, &new_node) != LY_SUCCESS) {
             ERRINFO(&err_info, plg_name, SR_ERR_LY, "lyd_new_opaq()", "");
             goto cleanup;
         }
