@@ -200,7 +200,7 @@ sr_shmmod_fill(sr_shm_t *shm_mod, size_t shm_mod_idx, const struct lyd_node *sr_
     sr_mod_t *smod;
     struct lyd_node *sr_child;
     off_t *shm_features;
-    const char *name;
+    const char *name, *rev;
     char *shm_end;
     size_t feat_i, feat_names_len, ds_plugin_names_len, old_shm_size;
     sr_datastore_t ds;
@@ -220,7 +220,9 @@ sr_shmmod_fill(sr_shm_t *shm_mod, size_t shm_mod_idx, const struct lyd_node *sr_
             name = lyd_get_value(sr_child);
         } else if (!strcmp(sr_child->schema->name, "revision")) {
             /* copy revision */
-            strcpy(smod->rev, lyd_get_value(sr_child));
+            rev = lyd_get_value(sr_child);
+            assert(strlen(rev) < sizeof(smod->rev));
+            strcpy(smod->rev, rev);
         } else if (!strcmp(sr_child->schema->name, "replay-support")) {
             /* set replay-support flag */
             smod->replay_supp = 1;
@@ -870,29 +872,37 @@ sr_shmmod_del_module_sess_oper_data(sr_conn_ctx_t *conn, const struct lys_module
     const struct sr_ds_handle_s *oper_ds_handle;
     struct lyd_node *mod_diff = NULL, *node;
 
-    /* get DS handle */
-    if ((err_info = sr_ds_handle_find(conn->mod_shm.addr + shm_mod->plugins[SR_DS_OPERATIONAL], conn,
-            &oper_ds_handle))) {
-        goto cleanup;
-    }
-
     if (has_data) {
-        /* load oper data */
-        if ((err_info = oper_ds_handle->plugin->load_cb(ly_mod, SR_DS_OPERATIONAL, cid, sid, NULL, 0, oper_ds_handle->plg_data,
-                &mod_diff))) {
+        /* get DS handle */
+        if ((err_info = sr_ds_handle_find(conn->mod_shm.addr + shm_mod->plugins[SR_DS_OPERATIONAL], conn,
+                &oper_ds_handle))) {
             goto cleanup;
         }
 
-        /* make into diff */
-        LY_LIST_FOR(mod_diff, node) {
-            if ((err_info = sr_diff_set_oper(node, "delete"))) {
+        /* create a diff only if the plugin requires it to store new data */
+        if (oper_ds_handle->plugin->oper_store_require_diff) {
+            /* load oper data */
+            if ((err_info = oper_ds_handle->plugin->load_cb(ly_mod, SR_DS_OPERATIONAL, cid, sid, NULL, 0, oper_ds_handle->plg_data,
+                    &mod_diff))) {
                 goto cleanup;
             }
+
+            /* make into diff */
+            LY_LIST_FOR(mod_diff, node) {
+                if ((err_info = sr_diff_set_oper(node, "delete"))) {
+                    goto cleanup;
+                }
+            }
         }
 
         /* store the empty data */
-        if ((err_info = oper_ds_handle->plugin->store_cb(ly_mod, SR_DS_OPERATIONAL, cid, sid, mod_diff, NULL,
-                oper_ds_handle->plg_data))) {
+        if ((err_info = oper_ds_handle->plugin->store_prepare_cb(ly_mod, SR_DS_OPERATIONAL, cid, sid, mod_diff,
+                NULL, oper_ds_handle->plg_data))) {
+            goto cleanup;
+        }
+
+        if ((err_info = oper_ds_handle->plugin->store_commit_cb(ly_mod, SR_DS_OPERATIONAL, cid, sid, mod_diff,
+                NULL, oper_ds_handle->plg_data))) {
             goto cleanup;
         }
     }
@@ -954,7 +964,7 @@ sr_shmmod_del_module_oper_data(sr_conn_ctx_t *conn, const struct lys_module *ly_
     SR_CHECK_MEM_GOTO(!oper_push_l, err_info, cleanup_unlock);
 
     /* EXT READ LOCK */
-    if ((err_info = sr_shmext_conn_remap_lock(conn, SR_LOCK_READ, 1, __func__))) {
+    if ((err_info = sr_shmext_conn_remap_lock(conn, SR_LOCK_READ, 0, __func__))) {
         goto cleanup_unlock;
     }
 
@@ -962,7 +972,7 @@ sr_shmmod_del_module_oper_data(sr_conn_ctx_t *conn, const struct lys_module *ly_
     memcpy(oper_push_l, conn->ext_shm.addr + shm_mod->oper_push_data, oper_push_count * sizeof *oper_push_l);
 
     /* EXT READ UNLOCK */
-    sr_shmext_conn_remap_unlock(conn, SR_LOCK_READ, 1, __func__);
+    sr_shmext_conn_remap_unlock(conn, SR_LOCK_READ, 0, __func__);
 
     for (i = 0; i < oper_push_count; ++i) {
         if (dead_only && sr_conn_is_alive(oper_push_l[i].cid)) {
@@ -1725,7 +1735,11 @@ sr_shmmod_copy_mod(const struct lys_module *ly_mod, const struct sr_ds_handle_s
     }
 
     /* write data to target */
-    if ((err_info = tds_handle->plugin->store_cb(ly_mod, tds, 0, 0, mod_diff, s_mod_data, tds_handle->plg_data))) {
+    if ((err_info = tds_handle->plugin->store_prepare_cb(ly_mod, tds, 0, 0, mod_diff, s_mod_data, tds_handle->plg_data))) {
+        goto cleanup;
+    }
+
+    if ((err_info = tds_handle->plugin->store_commit_cb(ly_mod, tds, 0, 0, mod_diff, s_mod_data, tds_handle->plg_data))) {
         goto cleanup;
     }
 
@@ -1851,69 +1865,3 @@ sr_shmmod_change_prio(sr_conn_ctx_t *conn, const struct lys_module *ly_mod, sr_d
 
     return NULL;
 }
-
-sr_error_info_t *
-sr_shmmod_session_oper_order(sr_session_ctx_t *session, const struct lys_module *ly_mod, uint32_t order,
-        uint32_t *order_p)
-{
-    sr_error_info_t *err_info = NULL;
-    sr_mod_shm_t *mod_shm;
-    sr_mod_t *shm_mod;
-    uint16_t i;
-
-    assert(!order || !order_p);
-
-    if (ly_mod) {
-        /* we have the module */
-        shm_mod = sr_shmmod_find_module(SR_CONN_MOD_SHM(session->conn), ly_mod->name);
-        SR_CHECK_INT_GOTO(!shm_mod, err_info, cleanup);
-
-        if (order_p) {
-            if ((err_info = sr_shmext_oper_push_get(session->conn, shm_mod, ly_mod->name, session->sid, order_p, NULL,
-                    SR_LOCK_NONE))) {
-                goto cleanup;
-            }
-        } else {
-            /* order 0 means it should be set to the next lowest order */
-            if ((err_info = sr_shmext_oper_push_update(session->conn, shm_mod, ly_mod->name, session->sid, order, -1,
-                    SR_LOCK_NONE))) {
-                goto cleanup;
-            }
-        }
-    } else {
-        mod_shm = SR_CONN_MOD_SHM(session->conn);
-
-        /* EXT READ LOCK */
-        if ((err_info = sr_shmext_conn_remap_lock(session->conn, SR_LOCK_READ, 1, __func__))) {
-            goto cleanup;
-        }
-
-        /* update all the modules with any push oper data */
-        for (i = 0; i < mod_shm->mod_count; ++i) {
-            shm_mod = SR_SHM_MOD_IDX(mod_shm, i);
-
-            if (!shm_mod->oper_push_data_count) {
-                continue;
-            }
-
-            if (order_p) {
-                if ((err_info = sr_shmext_oper_push_get(session->conn, shm_mod, ((char *)mod_shm) + shm_mod->name,
-                        session->sid, order_p, NULL, SR_LOCK_NONE))) {
-                    goto unlock;
-                }
-            } else {
-                if ((err_info = sr_shmext_oper_push_update(session->conn, shm_mod, ((char *)mod_shm) + shm_mod->name,
-                        session->sid, order, -1, SR_LOCK_NONE))) {
-                    goto unlock;
-                }
-            }
-        }
-
-unlock:
-        /* EXT READ UNLOCK */
-        sr_shmext_conn_remap_unlock(session->conn, SR_LOCK_READ, 1, __func__);
-    }
-
-cleanup:
-    return err_info;
-}
