@@ -25,7 +25,7 @@
 #include "common_types.h"
 #include "sysrepo_types.h"
 
-#define SR_SHM_VER 18   /**< Main, mod, and ext SHM version of their expected content structures. */
+#define SR_SHM_VER 19   /**< Main, mod, and ext SHM version of their expected content structures. */
 #define SR_MAIN_SHM_LOCK "sr_main_lock"     /**< Main SHM file lock name. */
 
 /**
@@ -146,8 +146,10 @@ typedef struct {
     off_t inv_deps;             /**< Array of inverse module data dependencies (off_t *) (offset in mod SHM). */
     uint16_t inv_dep_count;     /**< Number of inverse module data dependencies. */
 
-    off_t oper_push_data;       /**< Array of oper push data entries (offset in ext SHM). */
-    uint32_t oper_push_data_count;  /**< Number of oper poush data entries. */
+    off_t oper_push_data;       /**< Array of oper push data entries (offset in ext SHM), protected by operational DS
+                                     mod data locks. */
+    uint32_t oper_push_data_count;  /**< Number of oper push data entries, also protected by operational DS
+                                     mod data locks */
 
     struct {
         sr_rwlock_t lock;       /**< Process-shared lock for reading or preventing changes (READ) or modifying (WRITE)
@@ -190,7 +192,8 @@ typedef struct {
 typedef struct {
     uint32_t shm_ver;           /**< Main and ext SHM version of all expected data stored in them. Is increased with
                                      every change of their structure content (ABI change). */
-    pthread_mutex_t ext_lock;   /**< Process-shared lock for accessing holes and truncating ext SHM. */
+    pthread_mutex_t ext_lock;   /**< Process-shared lock for accessing holes and truncating ext SHM. Accessing ext SHM
+                                     structures requires another dedicated lock. */
 
     sr_rwlock_t context_lock;   /**< Process-shared lock for accessing connection LY context, lydmods data,
                                      and SHM mod modules. */
@@ -201,6 +204,7 @@ typedef struct {
     ATOMIC_T new_sr_sid;        /**< SID for a new session. */
     ATOMIC_T new_sub_id;        /**< Subscription ID of a new subscription. */
     ATOMIC_T new_evpipe_num;    /**< Event pipe number for a new subscription. */
+    ATOMIC_T new_operation_id;  /**< Operation ID to use for each callback of every operation. */
 
     char repo_path[256];        /**< Repository path used when main SHM was created. */
 } sr_main_shm_t;
@@ -367,6 +371,7 @@ typedef struct {
 
     ATOMIC_T priority;          /**< Priority of the subscriber. */
     uint32_t subscriber_count;  /**< Number of subscribers to process this event. */
+    uint32_t operation_id;      /**< Operation ID for the callback. */
 } sr_sub_shm_t;
 
 #endif /* _SHM_TYPES_H */
