@@ -1675,7 +1675,7 @@ cleanup:
 }
 
 sr_error_info_t *
-sr_nacm_check_operation(const char *nacm_user, const struct lyd_node *data, struct sr_denied *denied)
+sr_nacm_check_op(const char *nacm_user, const struct lyd_node *data, struct sr_denied *denied)
 {
     sr_error_info_t *err_info = NULL;
     const struct lyd_node *op = NULL;
@@ -1793,6 +1793,43 @@ cleanup:
     return err_info;
 }
 
+API int
+sr_nacm_check_operation(sr_session_ctx_t *session, const struct lyd_node *op)
+{
+    sr_error_info_t *err_info = NULL;
+    struct sr_denied denied = {0};
+
+    SR_CHECK_ARG_APIRET(!session || !op, session, err_info);
+
+    if (!session->nacm_user) {
+        goto cleanup;
+    }
+
+    /* check the operation */
+    if ((err_info = sr_nacm_check_op(session->nacm_user, op, &denied))) {
+        goto cleanup;
+    }
+
+    if (denied.denied) {
+        /* access denied */
+        if (denied.rule_name) {
+            sr_log(SR_LL_ERR, "NACM access denied by the rule \"%s\".", denied.rule_name);
+        } else if (denied.def) {
+            sr_log(SR_LL_ERR, "NACM access denied by \"%s\" node extension \"%s\".", LYD_NAME(denied.node), denied.def->name);
+        } else {
+            sr_log(SR_LL_ERR, "NACM access denied by the default NACM permissions.");
+        }
+
+        sr_errinfo_new_nacm(&err_info, "protocol", "access-denied", NULL, denied.node,
+                "Executing the operation is denied because \"%s\" NACM authorization failed.", session->nacm_user);
+        goto cleanup;
+    }
+
+cleanup:
+    free(denied.rule_name);
+    return sr_api_ret(session, err_info);
+}
+
 /**
  * @brief Filter out any nodes in a subtree for which the user does not have R access, recursively.
  *
@@ -1975,6 +2012,7 @@ sr_nacm_get_node_set_read_filter(sr_session_ctx_t *session, struct ly_set *set)
     char **groups = NULL;
     uint32_t group_count = 0;
     struct ly_set denied_set = {0};
+    struct lyd_node *node;
     uint32_t i, j;
     int denied;
 
@@ -1999,9 +2037,12 @@ sr_nacm_get_node_set_read_filter(sr_session_ctx_t *session, struct ly_set *set)
 
         denied = 0;
         for (j = 0; j < denied_set.count; ++j) {
-            if (denied_set.dnodes[j] == set->dnodes[i]) {
-                denied = 1;
-                break;
+            /* any parent could have been denied instead of the node */
+            for (node = set->dnodes[i]; node; node = lyd_parent(node)) {
+                if (denied_set.dnodes[j] == node) {
+                    denied = 1;
+                    break;
+                }
             }
         }
         ly_set_erase(&denied_set, NULL);
